@@ -72,8 +72,6 @@ with NiceImporter('xdg', 'python-xdg', '0.15'):
     import xdg  # NOQA
 with NiceImporter('requests', 'python-requests', '2.2.1'):
     import requests  # NOQA
-with NiceImporter('pynotify', 'python-notify', '0.1.1'):
-    import pynotify  # NOQA
 with NiceImporter('PyQt4.QtCore', 'PyQt4', '4.9.1'):
     import PyQt4.QtCore  # NOQA
 with NiceImporter('defer', 'python-defer', '1.0.6'):
